#include <genieArduino.h>
Genie genie;

// Voltmeter Demo
// 4D Systems
// This Demo communicates with a 4D Systems Display, configured with ViSi-Genie.
// The display has an angular meter, a LED digits, a rocker switch, and a static text.
// The program sends digital voltage readings to the LED digits and angular meter of the display module.
// The digital voltage readings are derived from pin A0 using ADC.Analog input voltage range is 0 to 5 volts only.
// The rocker switch of the display module is used for "turning on" the application.
// First, the program reads from the rocker switch manually using the Read Object function.
// If the rocker switch is on, the computed digital voltage readings are sent to the angular meter and LED digits.
// If the rocker switch is off, the angular meter and LED digits are made to display zero volts.

// This appication comes with a documentation: 4D-AN-P4019-ViSi-Genie A Simple Digital Voltmeter Application using an Arduino Host
// Download it here: http://www.4dsystems.com.au/appnotes/
/*
NOTE: Some Arduino boards such as the Due operate at 3.3 volts and tolerate analog input voltage levels from ground up to 3.3 V only. 
Applying more than 3.3 V may damage the chip. If attempting to use 3.3V Arduino boards for this application, make sure to limit the 
analog input voltage to 3.3 volts only and please read carefully your board’s datasheet. 
*/

int flag = 0;        //holds the "power status" of the voltmeter. flag = 0 means voltmeter is "off", flag = 1 means the voltmeter is "on".
int sensorPin = A0;  //the wiper of the trimmer potentiometer is connected to pin A0
int voltMeter;       //holds the digital voltage value to be sent to the angular meter
int voltLED;         //holds the digital voltage value to be sent to the LED digits

void setup() 
{ 
  Serial.begin(9600);
  // a few options to talk to the Display, uncomment the one you want
  // make sure that the ViSi Genie program has the same baud rate
  genie.Begin(Serial);  //Serial0
  genie.AttachEventHandler(myGenieEventHandler);

  //Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  pinMode(4, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(4, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(4, 0);  // unReset the Display via D4
  
  delay (3500);        //let the display start up
}

void loop() 
{ 
  static long waitPeriod = millis();  

  genie.DoEvents();        //Genie events are received and queued here.
  
  if (millis() >= waitPeriod) 
  {
    if(flag == 1){                                                
      voltLED = ((5.0 * analogRead(sensorPin) * 1000.0) / 1023);
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, voltLED);      //write to Leddigits0 the value of voltLED 
      voltMeter = voltLED/100;
      genie.WriteObject(GENIE_OBJ_ANGULAR_METER, 0x00, voltMeter); //write to Angularmeter0 the value of voltMeter     
    }
    
    else if(flag == 0){
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, 0); 
      genie.WriteObject(GENIE_OBJ_ANGULAR_METER, 0x00, 0);
    }
    
    // Do a manual read from Rockerswitch0. The results of this call will    
    // be available to myGenieEventHandler() after the display has responded
    genie.ReadObject(GENIE_OBJ_ROCKERSW, 0x00);
    waitPeriod = millis() + 50;
  }
}


/////////////////////////////////////////////////////////////////////
//
// This is the user's event handler. It is called by genieDoEvents() 
// when the following conditions are true
//
//		The link is in an IDLE state, and
//		There is an event to handle
//
// The event can be either a REPORT_EVENT frame sent asynchrounously 
// from the display or a REPORT_OBJ frame sent by the display in
// response to a READ_OBJ request.
//

void myGenieEventHandler(void) 
{
  int rockersw_val;          //holds the status of Rockerswitch0
  genieFrame Event;    
  genie.DequeueEvent(&Event);
  //Evaluate the event
  if (genie.EventIs(&Event, GENIE_REPORT_OBJ, GENIE_OBJ_ROCKERSW, 0x00)){ //if the event is a REPORT_OBJ from Rockerswitch0
    rockersw_val = genie.GetEventData(&Event);  //extract the MSB and LSB values and pass them to rockersw_val
    if (rockersw_val == 0)                     //if Rockerswitch0 is off
      flag = 0;                                //"turn off" the voltmeter
    else if (rockersw_val == 1)                //if Rockerswitch0 is on
      flag = 1;                                //"turn on" the voltmeter
  }
}



